@@ -20,7 +20,7 @@
  * The implementation is heavily inspired by pg_stat_statements
  *
  * Copyright (c) 2014,2017 Ronan Dunklau
- * Copyright (c) 2018-2023, The Powa-Team
+ * Copyright (c) 2018-2024, The Powa-Team
  *-------------------------------------------------------------------------
  */
 #include <limits.h>
@@ -100,6 +100,11 @@ PG_MODULE_MAGIC;
 	LWLockRelease(lock); \
 	}
 
+#if PG_VERSION_NUM < 170000
+#define MyProcNumber MyBackendId
+#define ParallelLeaderProcNumber ParallelLeaderBackendId
+#endif
+
 #if PG_VERSION_NUM < 140000
 #define ParallelLeaderBackendId ParallelMasterBackendId
 #endif
@@ -526,7 +531,7 @@ pgqs_set_query_sampled(bool sample)
 
 	/* in worker processes we need to get the info from shared memory */
 	LWLockAcquire(pgqs->sampledlock, LW_EXCLUSIVE);
-	pgqs->sampled[MyBackendId] = sample;
+	pgqs->sampled[MyProcNumber] = sample;
 	LWLockRelease(pgqs->sampledlock);
 }
 #endif
@@ -547,7 +552,7 @@ pgqs_is_query_sampled(void)
 
 	/* in worker processes we need to get the info from shared memory */
 	PGQS_LWL_ACQUIRE(pgqs->sampledlock, LW_SHARED);
-	sampled = pgqs->sampled[ParallelLeaderBackendId];
+	sampled = pgqs->sampled[ParallelLeaderProcNumber];
 	PGQS_LWL_RELEASE(pgqs->sampledlock);
 
 	return sampled;
@@ -731,6 +736,8 @@ pgqs_ExecutorEnd(QueryDesc *queryDesc)
 		HASHCTL		info;
 		pgqsEntry  *localentry;
 		HASH_SEQ_STATUS local_hash_seq;
+		/* We need to switch to the per-query memory context */
+		MemoryContext oldcxt = MemoryContextSwitchTo(queryDesc->estate->es_query_cxt);
 		pgqsWalkerContext *context = palloc(sizeof(pgqsWalkerContext));
 
 		context->queryId = queryDesc->plannedstmt->queryId;
@@ -851,6 +858,7 @@ pgqs_ExecutorEnd(QueryDesc *queryDesc)
 
 			PGQS_LWL_RELEASE(pgqs->lock);
 		}
+		MemoryContextSwitchTo(oldcxt);
 	}
 
 	if (prev_ExecutorEnd)
@@ -2195,7 +2203,6 @@ pg_qualstats_common(PG_FUNCTION_ARGS, pgqsVersion api_version,
 	}
 
 	PGQS_LWL_RELEASE(pgqs->lock);
-	tuplestore_donestoring(tupstore);
 	MemoryContextSwitchTo(oldcontext);
 
 	return (Datum) 0;
